#!/usr/bin/perl -w

# Andrei Popescu-Belis, ISSCO/TIM/ETI
# February 2007

# This program converts the XML / NXT annotation file of the AMI/IM2 corpus
# into database files, i.e. tab-separated files together with an sql script
# (for PostgreSQL) that allows the creation of an annotation database.

# The program makes use of separate XSLT stylesheets to create the tables 
# and summarizes the results into a log file named <timestamp>.log
# A unique procedure is defined, invoked for each annotation with a 
# different argument (some cases must be processed individually). 

# The following annotations (folders) are processed, with corresponding tables.
# These are based on the AMI Annotations release 1.2.2. of February 2nd, 2007
# (see <codings> and <coding-file> in AMI-metadata.xml)
#
#
# NAME (ext.)	PATH				STATUS
#  <agent-codings>				---
#wordalign		wordAlignment		n/a
#summ			participantSummaries	done
#feeltrace		emotion			n/a
#speech-quality	speechQuality		n/a
#segments		segments			done
#words		words				done
#foa			focus				done
#legs			legs				n/a
#pose			?				n/a
#dialog-act		dialogueActs		done
#head			headGesture			done
#hand			handGesture			done
#argumentstructs	argumentation/ae		done
#ne			namedEntities		done
#asr			ASR				n/a
#aseg			ASR				n/a
#  <interaction-codings>			---
#adjacency-pairs	dialogueActs		done
#ne-relations	namedEntities		n/a
#abssumm		abstractive			done
#extsumm		extractive			done
#summlink		extractive			done
#topic		topics			done
#argumentationrels argumentation/ar		done
#discussions	argumentation/dis		done


use strict ;
use warnings ;

sub convert_nxt2tsv($);		# converts each NXT annotation to TSV


# ------------------------------------------------------------------------
# Name of AMI annotations folder

my $drive_name = "";
if ($^O eq "MSWin32")   { $drive_name = "C:" } 
elsif ($^O eq "cygwin") { $drive_name = "C:" }	# (/c doesn't work with saxon)
else { print "Non tested OS, will try with no drive name.\n" }

my $root = "$drive_name/AMI-data-020207" ;
# this is the root of the AMI Annotation folder structure

my $db_folder = "$root/IM2DMA-annotation-database" ;
# this is the folder where this script and the stylesheets are located
# and where the TSV files will be created by this script as well


# ------------------------------------------------------------------------
#
# Declaration of path, extension, table creation and primary key info 
# for each annotation (this is the "annotation dictionary" or %dict)

my %dict = 
(
# ++++++++++++++++++ Language and discourse ++++++++++++++++++ 

words => {
	name => "Word transcription",
	extension => "words",
	folder => "words",
	table => "\n meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n item_id integer NOT NULL,\n annotator text NOT NULL,\n type text NOT NULL,\n content text,\n truncated text,\n mispronounced text,\n pronunciation text,\n starttime double precision,\n endtime double precision\n",
	pkey => "meeting_id, speaker_id, item_id, annotator",
	status => "done"
},
segments => {
	name => "Speech segments",
	extension => "segments",
	folder => "segments",
	table => "\n meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n segment_id integer NOT NULL,\n annotator text NOT NULL,\n starttime double precision,\n endtime double precision,\n channel text,\n startword integer,\n endword integer,\n words text\n",
	pkey => "meeting_id, speaker_id, segment_id, annotator",
	status => "done"
},
namedEntities => {
	name => "Named entities",
	extension => "ne",
	folder => "namedEntities",
	table => "\n meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n ne_id integer NOT NULL,\n annotator text NOT NULL,\n ne_type text NOT NULL,\n ne_abbrev text,\n ne_keystroke text,\n starttime double precision,\n endtime double precision,\n startword integer,\n endword integer,\n words text\n" , 
	pkey => "meeting_id, speaker_id, ne_id, annotator" ,
	status => "done"
},
topics => {
	name => "Topics",
	extension => "topic",
	folder => "topics",
	table => "meeting_id text NOT NULL,\n topic_id integer NOT NULL,\n annotator text NOT NULL,\n depth integer,\n starttime double precision,\n endtime double precision,\n scenario_topic_type text,\n other_description text\n" ,
	pkey => "meeting_id, topic_id, annotator",
	status => "done"
},
dialogueActs => {
	name => "Dialogue acts",
	extension => "dialog-act",
	folder => "dialogueActs",
	table => "\n meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n da_id integer NOT NULL,\n annotator text NOT NULL,\n da_name text,\n da_gloss text,\n starttime double precision,\n endtime double precision,\n startword integer,\n endword integer,\n words text\n" ,
	pkey => "meeting_id, speaker_id, da_id, annotator" ,
	status => "done"
},
adjacencyPairs => {
	name => "Adjacency pairs" ,
	extension => "adjacency-pairs",
	folder => "dialogueActs",
	table => "meeting_id text NOT NULL,\n annotator text NOT NULL,\n ap_id integer NOT NULL,\n ap_type text,\n ap_type_gloss text,\n source_speaker_id text,\n source_nxt_speaker text,\n source_annotator text,\n source_da_id integer,\n target_speaker_id text,\n target_nxt_speaker text,\n target_annotator text,\n target_da_id integer\n" ,
	pkey =>  "meeting_id, annotator, ap_id" ,
	status => "done"
},

# ++++++++++++++++++ Summaries ++++++++++++++++++ 

extractive => {
	name => "Extractive summaries",
	extension => "extsumm",
	folder => "extractive",
	table => "meeting_id text NOT NULL,\n annotator text NOT NULL,\n orig_pos integer,\n speaker_id text NOT NULL,\n nxt_speaker text,\n da_annotator text NOT NULL,\n da_id integer\n" , 
	pkey => "meeting_id, annotator, orig_pos, speaker_id, da_id" ,
	status => "done"
},
abstractive => {
	name => "Abstractive summaries",
	extension => "abssumm",
	folder => "abstractive",
	table => "meeting_id text NOT NULL,\n annotator text NOT NULL,\n paragraph_type text NOT NULL,\n sentence_number integer NOT NULL,\n sentence text\n" ,
	pkey => "meeting_id, annotator, sentence_number" ,
	status => "done"
},
summlink => {
	name => "Extractive-abstractive summaries links",
	extension => "summlink",
	folder => "extractive",
	table => "meeting_id text NOT NULL,\n annotator text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n da_annotator text NOT NULL,\n da_id integer NOT NULL,\n abs_annotator text NOT NULL,\n sentence_number integer NOT NULL\n" , 
	pkey => "meeting_id, annotator, speaker_id, da_annotator, da_id, abs_annotator, sentence_number" ,
	status => "done"
},
participantSummaries => {
	name => "Participant summaries",
	extension => "summ",
	folder => "participantSummaries",
	table => "meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n speaker_role text,\n paragraph_type text NOT NULL,\n sentence_number integer NOT NULL,\n sentence text\n" ,
	pkey => "meeting_id, speaker_id, sentence_number" ,
	status => "done"
},

# ++++++++++++++++++ Non-linguistic modalities ++++++++++++++++++ 

handGesture => {
	name => "Hand gesture",
	extension => "hand",
	folder => "handGesture",
	table => "meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n hand_gesture_id integer,\n annotator text NOT NULL,\n starttime double precision,\n endtime double precision,\n hand_gesture_type text\n" ,
	pkey => "meeting_id, speaker_id, hand_gesture_id, annotator" ,
	status => "done"
},
headGesture => {
	name => "Head gesture",
	extension => "head",
	folder => "headGesture",
	table => "meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n head_gesture_id integer,\n annotator text NOT NULL,\n starttime double precision,\n endtime double precision,\n head_gesture_type text,\n head_gesture_form text\n",
	pkey => "meeting_id, speaker_id, head_gesture_id, annotator",
	status => "done"
},
focus => {
	name => "Focus of attention",
	extension => "foa",
	folder => "focus",
	table => "meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n foa_id integer NOT NULL,\n annotator text NOT NULL,\n starttime double precision,\n endtime double precision,\n what_type text,\n what_id text,\n what_nxt text,\n what_role text\n" ,
	pkey => "meeting_id, speaker_id, foa_id, annotator" ,
	status => "done"
},

# ++++++++++++++++++ Argumentation ++++++++++++++++++ 

argumentstructs => {
	name => "Argumentation structures (units)",
	extension => "argumentstructs",
	folder => "argumentation/ae",
	table => "\n meeting_id text NOT NULL,\n speaker_id text NOT NULL,\n nxt_speaker text,\n annotator text NOT NULL,\n ae_id integer NOT NULL,\n ae_type text NOT NULL,\n ae_gloss text,\n starttime double precision,\n endtime double precision,\n startword integer,\n endword integer,\n words text\n" , 
	pkey => "meeting_id, speaker_id, ae_id, annotator" ,
	status => "done"
},
argumentationrels => {
	name => "Argumentation relations (between units)",
	extension => "argumentationrels",
	folder => "argumentation/ar",
	table => "\n meeting_id text NOT NULL,\n annotator text NOT NULL,\n ar_id integer NOT NULL,\n ar_type text NOT NULL,\n ar_gloss text,\n src_speaker_id text NOT NULL,\n src_nxt_speaker text,\n src_annotator text NOT NULL,\n src_id integer NOT NULL,\n trg_speaker_id text NOT NULL,\n trg_nxt_speaker text,\n trg_annotator text NOT NULL,\n trg_id integer NOT NULL\n" ,
	pkey => " meeting_id, annotator, ar_id, src_speaker_id, src_annotator, src_id, trg_speaker_id, trg_id ",
	status => "done"
},
discussions => {
	name => "Discussion units",
	extension => "discussions",
	folder => "argumentation/dis",
	table => "\n meeting_id text NOT NULL,\n annotator text NOT NULL,\n discussion_id integer NOT NULL,\n discussion_name text,\n sync_speaker_id text NOT NULL,\n sync_nxt_speaker text,\n sync_annotator text NOT NULL,\n sync_id integer NOT NULL",
	pkey => "meeting_id, annotator, discussion_id, sync_speaker_id, sync_annotator, sync_id",
	status => "done"
}

# ++++++++++++++++++ End annotation dictionary ++++++++++++++++++ 
);


# ------------------------------------------------------------------------
# Create and open LOGFILE and SQLFILE (database loader file)

my $d = 'nodate' ; # could also use "localtime"
if    ($^O eq "MSWin32") {$d = `date /T` ; chop $d ; chop $d ; my $t = `time /T` ; $t =~ s/\:/\./g ; chop $t ; $d = "$d-$t" }   
elsif (($^O eq "cygwin") || ($^O eq "solaris"))  {$d = `date +%y%m%d-%H%M%S`}
else  { print "Unknown OS: will use default for date-time.\n" ; $d = "default" }
chomp $d ;

open(LOGFILE,">$db_folder/$d.log") || die "\nCannot create log file: $db_folder/$d.log\n" ;

open(SQLFILE,">$db_folder/db_loader.sql")    || die "\nCannot create SQL file: $db_folder/db_loader.sql\n" ;

print STDOUT "Log file: $db_folder/$d.log\n" ;
print STDOUT "SQL file: $db_folder/db_loader.sql\n\n" ;

select SQLFILE ;


# ------------------------------------------------------------------------
# Write the headers of SQLFILE

print "--
-- SQL loader for AMI/IM2 database
-- Generated automatically at $d
--
-- To create the tables, connect to a database
-- and run this file within an SQL interpreter.
-- For PostgreSQL, execute initdb, postmaster, createdb,
-- then connect with psql and run \"\\i db_loader.sql\"
--
-- CAUTION: this will erase existing tables with same names.\n\n" ;

# ------------------------------------------------------------------------
# Call sub to create each table

#convert_nxt2tsv('words');
#convert_nxt2tsv('segments');
#convert_nxt2tsv('namedEntities');
#convert_nxt2tsv('topics');
#convert_nxt2tsv('dialogueActs');
#convert_nxt2tsv('adjacencyPairs');
#
#convert_nxt2tsv('handGesture');
#convert_nxt2tsv('headGesture');
#convert_nxt2tsv('focus');
#
#convert_nxt2tsv('extractive');
#convert_nxt2tsv('abstractive');
#convert_nxt2tsv('participantSummaries');
#convert_nxt2tsv('summlink');
#
#convert_nxt2tsv('argumentstructs');
#convert_nxt2tsv('discussions');
#convert_nxt2tsv('argumentationrels');

# Or go through all available (done) annotations in the dictionary:

foreach my $ann (keys %dict) {if ($dict{$ann}{status} eq "done") {convert_nxt2tsv($ann)}}


# ------------------------------------------------------------------------

close LOGFILE ;
close SQLFILE ;

# ------------------------------------------------------------------------
# End main program.


# ------------------------------------------------------------------------
# Sub for processing annotations: same procedure for each one (apply XSLT
# and write table creation script) but different stylesheets and tables,
# and some particular cases require specific processing.
# ------------------------------------------------------------------------


sub convert_nxt2tsv($) {

my $anno = $_[0] ; 

my $extension = $dict{$anno}{extension}; 

print STDOUT "Processing: $dict{$anno}{name} ..." ;
print LOGFILE "\nProcessing: $dict{$anno}{name} \n" ;

open(ANNOFILE,">$db_folder/$anno.tsv") 
     || die "\nCannot create log file: $db_folder/$anno.tsv\n" ;

my $anno_folder_esc  =  "$root/$dict{$anno}{folder}" ;
   $anno_folder_esc  =~ s/\ /\\\ /g ;

	# Select only scenario meetings (?S*.$extension.xml => ES,IS,TS)

my @anno_filelist = glob("$anno_folder_esc/IS1000*.$extension.xml") ;

	# Globbing requires whitespaces to be escaped with a \ hence $anno_folder_esc

foreach my $longname (@anno_filelist) { $longname =~ s/^.*\/([\w\.]*)\.$extension\.xml$/$1/ }

	# For debugging, use a very short list, e.g.: 
	# @anno_filelist = ('IS1008b', 'EN2004a',) ;

my $buffer ;

foreach my $shortname (@anno_filelist) {

	$buffer = `java -cp saxon.jar com.icl.saxon.StyleSheet
			"$root/$dict{$anno}{folder}/$shortname.$extension.xml" 
			$anno.xslt 
			wordfile="file:/$root/words/$shortname.words.xml" 
			speakerfile="file:/$root/corpusResources/meetings.xml"
			datypesfile="file:/$root/ontologies/da-types.xml"
			netypesfile="file:/$root/ontologies/ne-types.xml"
			aetypesfile="file:/$root/ontologies/ae-types.xml"
			aptypesfile="file:/$root/ontologies/ap-types.xml"
			artypesfile="file:/$root/ontologies/ar-types.xml"
			wordfilepath="file:/$root/words/"
			defaulttopicsfile="file:/$root/ontologies/default-topics.xml"
			segmentspath="file:/$root/segments/"` ;

			# 1. This can be replaced by a call to any XSLT processor available.
			# 2. It looks like $anno_folder_esc cannot be used here (on Win32). 
			# 3. Not all of these arguments are really used every time.

			# For saxon.exe version: use only $buffer = `saxon... and no '-cp' 
			#
			# For Solaris, put everything on one line and remove 'file:/'.
			# For JDK 1.3 use '-cp saxon.jar:saxon-xml-apis.jar' (note the ':')
			# but for JDK 1.5 and higher '-cp saxon.jar' is enough.
			#
			# $buffer = `java -cp saxon.jar:saxon-xml-apis.jar com.icl.saxon.StyleSheet "$root/$dict{$anno}{folder}/$shortname.$extension.xml" $anno.xslt wordfile="$root/words/$shortname.words.xml" speakerfile="$root/corpusResources/meetings.xml" datypesfile="$root/ontologies/da-types.xml" netypesfile="$root/ontologies/ne-types.xml" aetypesfile="$root/ontologies/ae-types.xml" aptypesfile="$root/ontologies/ap-types.xml" artypesfile="$root/ontologies/ar-types.xml" wordfilepath="$root/words/" defaulttopicsfile="$root/ontologies/default-topics.xml" segmentspath="$root/segments/"` ;



	print ANNOFILE $buffer ;
	print LOGFILE "Processed file: $shortname.$extension.xml\n" ;
	print STDOUT "$shortname " ;
}

close ANNOFILE ;
my $n = $#anno_filelist + 1 ;
print LOGFILE "Processed $anno from $n files\n" ;


print "--
-- Create/load table: $anno
--

DROP TABLE $anno;

CREATE TABLE $anno( $dict{$anno}{table} );

COPY $anno FROM \'$db_folder/$anno.tsv\' using delimiters \'\\t\' ;

ALTER TABLE ONLY $anno ADD CONSTRAINT $anno\_pkey PRIMARY KEY ( $dict{$anno}{pkey} );

" ;

print LOGFILE "Wrote SQL creation instructions for $anno table on db_loader.sql\n" ;
print STDOUT " done\n" ;
}
# ------------------------------------------------------------------------
# End definition of  sub convert_nxt2tsv($) 
# ------------------------------------------------------------------------
